// Copyright (c) 2015 HigPen Works (Kenichiro Matsuura / Yuki Tsukasa). All rights reserved.
#pragma once
#include "Config.h"
#include "Hit.h"
#include "Input.h"
#include "Math.h"
#include "Mover.h"
#include "Shader.h"
#include "Texture.h"
#include "Sound.h"
#include <ctime>
#include <map>
using namespace std;

class CGame {
	friend class CMover;
	CShader* Shader;
	GLKMatrix4 ViewProjection;
	map<wstring, CTexture*> Texture;
	map<wstring, CSound*> Sound;
public:
	CGame();
	virtual ~CGame();
	virtual void Reset();
	virtual void Move();
	virtual void Draw();
	void Clear(const CColor& color);
	GLKMatrix4 GetViewProjection() { return ViewProjection; }
	CTexture* GetTexture(wstring name);
	CSound* GetSound(wstring name);
	CMover* Camera;
	CVector Scale;
};

extern CGame* Game;
CGame* Main();

inline CTexture* GetTexture(wstring name) { return Game->GetTexture(name); }
inline CSound* GetSound(wstring name) { return Game->GetSound(name); }

inline float Random(float from, float to) { return float(from+(to-from)*double(rand())/RAND_MAX); }
inline float RandomLess(float from, float to) { return float(from+(to-from)*double(rand())/(double(RAND_MAX)+1)); }
inline void RandomSeed() { srand((unsigned)time(NULL)); }
