// Copyright (c) 2015 HigPen Works (Kenichiro Matsuura / Yuki Tsukasa). All rights reserved.
#include "Game.h"
#include <string>
using namespace std;

CGame* Game;

CTouch Touch[MAX_TOUCH], OldTouch[MAX_TOUCH], NewTouch[MAX_TOUCH];
CVector Tilt;

CConfig Config;

CGame::CGame() : Shader(new CShader()), Scale(1, 1.5f), Camera(0) {
}

CGame::~CGame() {
	for (map<wstring, CTexture*>::iterator i=Texture.begin(); i!=Texture.end(); i++) delete i->second;
	for (map<wstring, CSound*>::iterator i=Sound.begin(); i!=Sound.end(); i++) delete i->second;
	delete Shader;
}

void CGame::Reset() {
	Reset();
}

void CGame::Move() {
	for (int i=0; i<MAX_TOUCH; i++) {
		OldTouch[i]=Touch[i], Touch[i]=NewTouch[i];
		if (NewTouch[i]>0) NewTouch[i].Time++;
	}
	Move();
}

void CGame::Draw() {
	GLKMatrix4 mv=(-Camera->Position).MatrixPosition();
	mv=GLKMatrix4Multiply(Camera->Rotation.Inverse().MatrixRotation(), mv);
	GLKMatrix4 mp=GLKMatrix4MakeOrtho(-Scale.X, Scale.X, -Scale.Y, Scale.Y, 0, 1);
	ViewProjection=GLKMatrix4Multiply(mp, mv);
	Shader->Begin();
	Draw();
	Shader->End();
}

void CGame::Clear(const CColor& color) {
	glClearColor(color.R, color.G, color.B, color.A);
	glClear(GL_COLOR_BUFFER_BIT);
}

CTexture* CGame::GetTexture(wstring name) {
	map<wstring, CTexture*>::iterator i=Texture.find(name);
	if (i!=Texture.end()) {
		return i->second;
	} else {
		CTexture* t=new CTexture(name);
		Texture.insert(pair<wstring, CTexture*>(name, t));
		return t;
	}
}

CSound* CGame::GetSound(wstring name) {
	map<wstring, CSound*>::iterator i=Sound.find(name);
	if (i!=Sound.end()) {
		return i->second;
	} else {
		CSound* s=new CSound(name);
		Sound.insert(pair<wstring, CSound*>(name, s));
		return s;
	}
}

