// Copyright (c) 2015 HigPen Works (Kenichiro Matsuura / Yuki Tsukasa). All rights reserved.
#import "GameViewController.h"
#import <OpenGLES/ES2/glext.h>
#import <CoreMotion/CoreMotion.h>
#include <stdlib.h>
#include "Game.h"

@interface GameViewController () {}
@property (strong, nonatomic) EAGLContext* context;
@property (strong, nonatomic) CMMotionManager* motion;
@end

@implementation GameViewController

-(void)viewDidLoad {
	[super viewDidLoad];
	[[UIApplication sharedApplication] setStatusBarHidden:YES withAnimation:UIStatusBarAnimationNone];
	self.preferredFramesPerSecond=60;
	self.context=[[EAGLContext alloc] initWithAPI:kEAGLRenderingAPIOpenGLES2];
	if (!self.context) NSLog(@"Failed to create ES context");
	GLKView *view=(GLKView *)self.view;
	view.context=self.context;
	view.drawableDepthFormat=GLKViewDrawableDepthFormat24;
	[EAGLContext setCurrentContext:self.context];
	self.motion=[[CMMotionManager alloc] init];
	self.motion.deviceMotionUpdateInterval=0.016;
	self.motion.accelerometerUpdateInterval=0.016;
	[self.motion startDeviceMotionUpdatesToQueue:[NSOperationQueue currentQueue] withHandler:^(CMDeviceMotion* m, NSError* error) {
		CMAcceleration a=m.gravity;
		Tilt.X=a.x, Tilt.Y=a.y, Tilt.Z=a.z;
		switch ([[UIApplication sharedApplication] statusBarOrientation]) {
			case UIInterfaceOrientationPortraitUpsideDown: Tilt.X=-a.x, Tilt.Y=-a.y; break;
			case UIInterfaceOrientationLandscapeLeft: Tilt.X=a.y, Tilt.Y=-a.x; break;
			case UIInterfaceOrientationLandscapeRight: Tilt.X=-a.y, Tilt.Y=a.x; break;
			default: break;
		}
	}];
	Game=Main();
	Game->CGame::Reset();
}

-(void)dealloc {
	[self.motion stopAccelerometerUpdates];
	[EAGLContext setCurrentContext:self.context];
	delete Game;
	[EAGLContext setCurrentContext:nil];
}

-(void)didReceiveMemoryWarning {
	[super didReceiveMemoryWarning];
	if ([self isViewLoaded] && [[self view] window]==nil) {
		self.view = nil;
		[EAGLContext setCurrentContext:self.context];
		delete Game;
		[EAGLContext setCurrentContext:nil];
		self.context = nil;
	}
}

-(void)update {
	Game->CGame::Move();
}

- (void)glkView:(GLKView *)view drawInRect:(CGRect)rect {
	glDisable(GL_SCISSOR_TEST);
	int s=[UIScreen mainScreen].scale, w=rect.size.width*s, h=rect.size.height*s;
	glViewport(0, 0, w, h);
	Game->Clear(CColor(0, 0, 0, 1));
	float sx=Game->Scale.X, sy=Game->Scale.Y;
	int vw=h*sx/sy, vh=w*sy/sx;
	if (float(w)/h<sx/sy) {
		glViewport(0, (h-vh)/2, w, vh); 
		glScissor(0, (h-vh)/2, w, vh);
	} else {
		glViewport((w-vw)/2, 0, vw, h);
		glScissor((w-vw)/2, 0, vw, h);
	}
	glEnable(GL_SCISSOR_TEST);
	Game->CGame::Draw();
}

-(void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event {
	for (int i=0, n=(int)[touches count]; i<n; i++) NewTouch[i].Time=1;
	[self touchesMoved:touches withEvent:event];
}

-(void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event {
	for (int i=0; i<MAX_TOUCH; i++) NewTouch[i].Time=0;
}

-(void)touchesMoved:(NSSet*)touches withEvent:(UIEvent*)event {
	NSArray* a=[touches allObjects];
	int s=[UIScreen mainScreen].scale;
	GLint v[4];
	glGetIntegerv(GL_VIEWPORT, v);
	for (int i=0, n=(int)[touches count]; i<n; i++) {
		CGPoint p=[a[i] locationInView:[self view]];
		NewTouch[i].Position=CVector(
			((p.x*s-v[0])*2/v[2]-1)*Game->Scale.X,
			(1-(p.y*s-v[1])*2/v[3])*Game->Scale.Y);
	}
}

@end
