// Copyright (c) 2015 HigPen Works (Kenichiro Matsuura / Yuki Tsukasa). All rights reserved.
#include "Main.h"

//---------------------------------------------------------
class CStageNewObject : public CStage {
public:
	void Reset() {
		New<CPlayer>();
	}
};
CStage* StageNewObject() { return New<CStageNewObject>(); }

//---------------------------------------------------------
class CPlayerMoveObject : public CPlayer {
public:
	void Move() {
		Position.X+=0.01f;
	}
};
class CStageMoveObject : public CStage {
public:
	void Reset() {
		New<CPlayerMoveObject>();
	}
};
CStage* StageMoveObject() { return New<CStageMoveObject>(); }

//---------------------------------------------------------
class CStageTouch : public CStage {
public:
	void Move() {
		if (Touch[0]) New<CPlayer>(Touch[0].Position);
	}
};
CStage* StageTouch() { return New<CStageTouch>(); }

//---------------------------------------------------------
class CPlayerTouchObject : public CPlayer {
public:
	void Move() {
		if (Touch[0] && Length(Position-Touch[0].Position)<0.1f) Life=0;	
	}
};
class CStageTouchObject : public CStage {
public:
	void Reset() {
		for (int i=0; i<10; i++) {
			New<CPlayerTouchObject>(CVector(Random(-1, 1), Random(-1.5f, 1.5f)));
		}
	}
};
CStage* StageTouchObject() { return New<CStageTouchObject>(); }

//---------------------------------------------------------
class CPlayerHitObject : public CPlayer {
public:
	void Move() {
		Position.X+=0.02f;
		EnemyList.Apply([&](CMover* e) {
			if (Length(Position-e->Position)<0.1f) e->Life=0; 
		});
	}
};
class CStageHitObject : public CStage {
public:
	void Reset() {
		New<CPlayerHitObject>(CVector(-1, 0));
		New<CEnemy>(CVector(-0.3f, 0));
		New<CEnemy>(CVector(0.3f, 0));
	}
};
CStage* StageHitObject() { return New<CStageHitObject>(); }

//---------------------------------------------------------
class CEffectHitEffect : public CEffect {
public:
	void Move() {
		Scale*=1.05f;
		Color.A*=0.9f;
		if (Color.A<0.01f) Life=0; 
	}
};
class CPlayerHitEffect : public CPlayer {
public:
	void Move() {
		Position.X+=0.02f;
		EnemyList.Apply([&](CMover* e) {
			if (Length(Position-e->Position)<0.1f) {
				e->Life=0; 
				New<CEffectHitEffect>(e->Position);
			}
		});
	}
};
class CStageHitEffect : public CStage {
public:
	void Reset() {
		New<CPlayerHitEffect>(CVector(-1, 0));
		New<CEnemy>(CVector(-0.3f, 0));
		New<CEnemy>(CVector(0.3f, 0));
	}
};
CStage* StageHitEffect() { return New<CStageHitEffect>(); }

//---------------------------------------------------------
class CEnemyScore : public CEnemy {
public:
	void Move() {
		if (Touch[0] && Length(Position-Touch[0].Position)<0.1f) {
			Life=0;
			Score++;
			New<CEffectHitEffect>(Position);
		}
		Color.A*=0.99f;
		if (Color.A<0.05f) Life=0;
	}
};
class CStageScore : public CStage {
public:
	void Move() {
		UseScore=true;
		if (Random(0, 1)<0.1f) New<CEnemyScore>(CVector(Random(-1, 1), Random(-1.5f, 1.5f)));
	}
};
CStage* StageScore() { return New<CStageScore>(); }

