// Copyright (c) 2015 HigPen Works (Kenichiro Matsuura / Yuki Tsukasa). All rights reserved.
#include "Main.h"

//---------------------------------------------------------
class CPlayerWalk : public CPlayer {
public:
	void Move() {
		if (Touch[0]) OldPosition=Touch[0].Position;
		Position.Homing(OldPosition, 0.02f);
	}
};
class CStageWalk : public CStage {
public:
	void Reset() {
		New<CPlayerWalk>();
	}
};
CStage* StageWalk() { return New<CStageWalk>(); }

//---------------------------------------------------------
class CSword : public CWeapon {
	void Move() {
		Position=Owner->Position+CVector(0, 0.2f);
	}
};
class CPlayerSword : public CPlayerWalk {
public:
	void Reset() {
		CPlayerWalk::Reset();
		New<CSword>()->Owner=this;
	}
};
class CStageSword : public CStage {
public:
	void Reset() {
		New<CPlayerSword>();
	}
};
CStage* StageSword() { return New<CStageSword>(); }

//---------------------------------------------------------
class CShakeSword : public CWeapon {
	void Move() {
		Rotation=RotationZ(Sin(Time*0.02f)*0.15f);
		Position=Owner->Position+CVector(0, 0.2f)*Rotation;
		Time++;
	}
};
class CPlayerShakeSword : public CPlayerWalk {
public:
	void Reset() {
		CPlayerWalk::Reset();
		New<CShakeSword>()->Owner=this;
	}
};
class CStageShakeSword : public CStage {
public:
	void Reset() {
		New<CPlayerShakeSword>();
	}
};
CStage* StageShakeSword() { return New<CStageShakeSword>(); }

//---------------------------------------------------------
class CEnemyShakeSword : public CEnemy {
public:
	void Move() {
		WeaponList.Apply([&](CMover* m) {
			if (Length(m->Position-Position)<0.2f) Life=0;
		});
	}
};
class CStageEnemyShakeSword : public CStage {
public:
	void Reset() {
		New<CPlayerShakeSword>();
		for (int i=0; i<10; i++) New<CEnemyShakeSword>(CVector(Random(-1, 1), Random(-1.5f, 1.5f)));
	}
};
CStage* StageEnemyShakeSword() { return New<CStageEnemyShakeSword>(); }

//---------------------------------------------------------
class CPlayerMaze : public CPlayer {
public:
	void Reset() {
		CPlayer::Reset();
		OldPosition=Position;
	}
	void Move() {
		Game->Camera->Position=Position;
		if (Touch[0]) OldPosition=Game->Camera->Position+Touch[0].Position;
		Position.Homing(OldPosition, 0.02f);
		CHitBox(this, BlockList, 0.05f);
	}
};

class CStageMaze : public CStage {
public:
	void MakeMaze(const char* maze[]) {
		for (int y=0; maze[y]; y++) for (int x=0; maze[y][x]; x++) {
			CVector v(x*0.2f, -y*0.2f);
			switch (maze[y][x]) {
				case 'P': New<CPlayerMaze>(v); break;
				case 'E': New<CEnemy>(v); break;
				case 'I': New<CItem>(v); break;
				case '#': New<CBlock>(v); break;
			}
		}		
	}
	void Reset() {
		const char* maze[]={
			"#########",
			"#      I#",
			"# ### # #",
			"#   # # #",
			"# #   # #",
			"# # #   #",
			"# # ### #",
			"#P      #",
			"#########",
			0
		};
		MakeMaze(maze);
	}
};
CStage* StageMaze() { return New<CStageMaze>(); }

//---------------------------------------------------------
class CStageSwitchMaze : public CStageMaze {
public:
	void Reset() {
		const char* maze0[]={
			"#########",
			"#       #",
			"# ## ## #",
			"# #   # #",
			"# #   # #",
			"# # I # #",
			"# ##### #",
			"#P      #",
			"#########",
			0
		};
		const char* maze1[]={
			"#########",
			"#   #   #",
			"# # # # #",
			"# # # # #",
			"# # # # #",
			"# # # # #",
			"# # # # #",
			"#P#   #I#",
			"#########",
			0
		};
		Clear();
		if (Type==0) MakeMaze(maze0);
		if (Type==1) MakeMaze(maze1);
	}
	void Clear() {
		PlayerList.Kill();
		EnemyList.Kill();
		ItemList.Kill();
		BlockList.Kill();
		Game->Camera->Position=0;
	}
	void Move() {
		PlayerList.Apply([&](CMover* p) {
			ItemList.Apply([&](CMover* i) {
				if (Length(p->Position-i->Position)<0.2f) {
					Type=1-Type;
					Reset();
				}
			});
		});
	}
};
CStage* StageSwitchMaze() { return New<CStageSwitchMaze>(); }

//---------------------------------------------------------
class CStageEncounterRandom : public CStageSwitchMaze {
public:
	void Reset() {
		const char* maze[]={
			"#########",
			"#       #",
			"#   #   #",
			"#       #",
			"# # # # #",
			"#       #",
			"#   #   #",
			"#P      #",
			"#########",
			0
		};
		Clear();
		if (Type==0) MakeMaze(maze);
		if (Type==1) New<CEnemy>()->Scale=0.7f;
	}
	void Move() {
		if (Type==0 && Touch[0] && Random(0, 1)<0.01f) {
			PlayerList.Apply([&](CMover* m) { Position=m->Position; });
			Type=1, Reset();
		}
		if (Type==1 && !OldTouch[0] && Touch[0]) {
			Type=0, Reset();
			PlayerList.Apply([&](CMover* m) { m->Position=Position; });
		}
		if (Type==1) {
			EnemyList.Apply([&](CMover* m) { m->Rotation=RotationZ(Sin(Time*0.02f)*0.1f); });
			Time++;
		}
	}
};
CStage* StageEncounterRandom() { return New<CStageEncounterRandom>(); }

//---------------------------------------------------------
class CStageEncounterSymbol : public CStageSwitchMaze {
public:
	void Reset() {
		const char* maze[]={
			"#########",
			"#       #",
			"# E # E #",
			"#       #",
			"# # # # #",
			"#       #",
			"# E # E #",
			"#P      #",
			"#########",
			0
		};
		Clear();
		if (Type==0) {
			MakeMaze(maze);
			EnemyList.Apply([&](CMover* e) {
				if (Length(e->Position-Position)<0.2f) e->Life=0;
			});
		}
		if (Type==1) New<CEnemy>()->Scale=0.7f;
	}
	void Move() {
		if (Type==0) {
			PlayerList.Apply([&](CMover* p) {
				EnemyList.Apply([&](CMover* e) {
					if (e->Life>0 && Length(e->Position-p->Position)<0.2f) {
						Position=p->Position;
						Type=1, Reset();
					}
				});
			});
		}
		if (Type==1 && !OldTouch[0] && Touch[0]) {
			Type=0, Reset();
			PlayerList.Apply([&](CMover* m) { m->Position=Position; });
		}
		if (Type==1) {
			EnemyList.Apply([&](CMover* m) { m->Rotation=RotationZ(Sin(Time*0.02f)*0.1f); });
			Time++;
		}
	}
};
CStage* StageEncounterSymbol() { return New<CStageEncounterSymbol>(); }

//---------------------------------------------------------
class CStageGauge : public CStage {
public:
	static const int MaxLife;
	void Reset() {
		Life=MaxLife/2;
		New<CBlock>()->Texture=GetTexture(L"White.png");
	}
	void Move() {
		if (Touch[0]) Life+=2; else Life--;
		Life=max(1, min(MaxLife, Life));
		BlockList.Apply([&](CMover* m) {
			m->Color=CColor(0, 0, 0, 1);
			m->Scale.X=(float)Life/MaxLife;
			m->Position.X=m->Scale.X-1;
		});
	}
};
CStage* StageGauge() { return New<CStageGauge>(); }
const int CStageGauge::MaxLife=100;

//---------------------------------------------------------
class CEffectCommand : public CEffect {
public:
	void Move() {
		Scale*=1.1f;
		Color.A*=0.98f;
		if (Color.A<0.01f) Life=0;
	}
};
class CStageCommand : public CStage {
public:
	static const int N=4;
	void Reset() {
		const wchar_t* texture[]={L"Player.png", L"Enemy.png", L"Weapon.png", L"Bullet.png"};
		for (int i=0; i<N; i++) New<CPlayer>(CVector(0, 0.45f-i*0.3f))->Texture=GetTexture(texture[i]);
	}
	void Move() {
		PlayerList.Apply([&](CMover* m) {			
			if (Length(m->Position-Touch[0].Position)<0.2f) {
				m->Color.A=1;
				if (OldTouch[0] && !Touch[0]) New<CEffectCommand>()->Texture=m->Texture;
			} else m->Color.A=0.5f;
		});
	}
};
CStage* StageCommand() { return New<CStageCommand>(); }

//---------------------------------------------------------
class CWeaponEquipment : public CWeapon {
public:
	void Move() {
		Rotation=RotationZ(Sin(Time*0.01f)*0.2f);
		Position=Owner->Position+CVector(0, Owner->Scale.Y+Scale.Y)*Rotation;
		Time++;
		if (!OldTouch[0] && Touch[0]) {
			ItemList.Apply([&](CMover* m) {
				if (Length(m->Position-Touch[0].Position)<0.2f) Texture=m->Texture;
			});
			BlockList.Apply([&](CMover* m) {
				CVector v=m->Position-Touch[0].Position, s=m->Scale;
				if (abs(v.X)<s.X && abs(v.Y)<s.Y) Scale.Y+=m->Type*0.1f;
			});
		}
		Scale.Y=max(0.1f, min(0.5f, Scale.Y));
	}
};
class CStageEquipment : public CStage {
public:
	static const int N=4;
	void Reset() {
		const wchar_t* texture[]={L"Weapon.png", L"Bullet.png", L"Effect.png", L"Item.png"};
		for (int i=0; i<N; i++) New<CItem>(CVector(i*0.3f-0.45f, -0.6f))->Texture=GetTexture(texture[i]);
		New<CWeaponEquipment>()->Owner=New<CPlayer>(CVector(0, 0.3f));
		for (int i=-1; i<=1; i+=2) {
			CBlock* b=New<CBlock>(i, CVector(0, i*0.3f-0.6f));
			b->Scale=CVector(0.6f, 0.15f);
			b->Color=CColor(0, 0, 0, 0.5f);
			b->Texture=GetTexture(L"White.png");
		}
	}
};
CStage* StageEquipment() { return New<CStageEquipment>(); }

